local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Omen")
local SML = AceLibrary("SharedMedia-1.0")
local SML_statusbar = SML:List("statusbar")

local formatOptions = {"%2.0f", "%2.1f", "%2.2f"}

local options = {
	type='group',
	args = {
		toggle = {
			type = "execute",
			desc = L["Toggle Omen"],
			name = L["Toggle Omen"],
			func = "Toggle",
			order = 102
		},
		showtest = {
			type = "execute",
			desc = L["Show Test Bars"],
			name = L["Show Test Bars"],
			func = "ShowTest"
		},		
		width = {
			type = "range",
			name = L["Width"],
			desc = L["Width"],
			step = 10,
			min = 150,
			max = 600,
			isPercent = false,
			get = function()
				return Omen.db.profile.Width
			end,
			set = function(v)
				Omen.db.profile.Width = v
				Omen:SetAnchorWidth(v)
			end
		},		
		scale = {
			type = "range",
			name = L["Scale"],
			desc = L["Scale"],
			step = 0.01,
			min = 0.25,
			max = 5.0,
			isPercent = false,
			get = function()
				return Omen.db.profile.Scale
			end,
			set = function(v)
				Omen.db.profile.Scale = v
				Omen.Anchor:SetScale(v)
			end
		},			
		toggle = {
			type = "toggle",
			name = L["Active When Solo"],
			desc = L["Activate Omen when solo or in a battlegroups (testing purposes only)"],
			get = function()
				return Omen.db.profile.ActivateSolo
			end,
			set = function(v)
				Omen.db.profile.ActivateSolo = v
				AceLibrary("Threat-1.0"):RequestActiveOnSolo(v)
			end
		},	
		showktm = {
			type = "toggle",
			name = L["Show KTM Data"],
			desc = L["Show KTM Data"],
			get = function()
				return Omen.db.profile.ShowKtmData
			end,
			set = function(v)
				Omen.db.profile.ShowKtmData = v
			end
		},			
		bars = {
			type = "group",
			name = L["Bars"],
			desc = L["Bars"],
			args = {
				animateBars = {
					type = "toggle",
					desc = L["Animate Bars"],
					name = L["Animate Bars"],
					get = function()
						return Omen.db.profile.AnimateBars
					end,
					set = function(t)
						Omen.db.profile.AnimateBars = t
					end
				},
				growUp = {
					type = "toggle",
					desc = L["Grow Upwards"],
					name = L["Grow Upwards"],
					get = function()
						return Omen.db.profile.GrowUp
					end,
					set = function(t)
						Omen.db.profile.GrowUp = t
						if t then
							local x, y = Omen.Anchor:GetLeft(), Omen.Anchor:GetBottom()
							Omen.Anchor:ClearAllPoints()
							Omen.Anchor:SetPoint("BOTTOM", UIParent, "CENTER", x, y)
						else
							local x, y = Omen.Anchor:GetLeft(), Omen.Anchor:GetTop()
							Omen.Anchor:ClearAllPoints()
							Omen.Anchor:SetPoint("TOP", UIParent, "CENTER", x, y)
						end
					end
				},				
				stretchBarTextures = {
					type = "toggle",
					desc = L["Stretch Bar Textures"],
					name = L["Stretch Bar Textures"],
					get = function()
						return Omen.db.profile.StretchBarTextures
					end,
					set = function(t)
						Omen.db.profile.StretchBarTextures = t
						if t then
							for k,v in pairs(Omen.bars) do
								v.tex:SetTexCoord(0, 1, 0, 1)
							end
						end
					end
				},
				maxBars = {
					type = "range",
					name = L["Number of bars"],
					desc = L["Number of bars"],
					step = 1,
					min = 1,
					max = 40,
					isPercent = false,
					get = function()
						return Omen.db.profile.MaxBars
					end,
					set = function(v)
						Omen.db.profile.MaxBars = v
					end
				},
				barHeight = {
					type = "range",
					name = L["Bar Height"],
					desc = L["Bar Height"],
					step = 1,
					min = 10,
					max = 100,
					isPercent = false,
					get = function()
					return Omen.db.profile.BarHeight
					end,
					set = function(h)
						Omen.db.profile.BarHeight = h
						local barCt = 0
						for k, v in pairs(Omen.bars) do
							v:SetHeight(h)
							if v.threatVal > 0 then
								barCt = barCt + 1
							end
						end
						Omen.Anchor:SetHeight(h * max(0,barCt) + Omen.barsOffset + 5)
					end
				},
				barTexture = {
					type = "text",
					name = L["Bar Texture"],
					desc = L["Bar Texture"],
					get = function() return Omen.db.profile.BarTexture end,
					set = function(tex)
						Omen.db.profile.BarTexture = tex
						for k, v in pairs(Omen.bars) do
							v.tex:SetTexture(SML:Fetch("statusbar", tex))
						end
					end,
					validate = SML_statusbar,
					usage = "<texture>",
				}		
			}
		},
		tps = {
			type = "group",
			name = L["Threat Per Second"],
			desc = L["Threat Per Second"],
			args = {
				tpsUpdateFrequency = {
					type = "range",
					name = L["Update Frequency"],
					desc = L["Update Frequency"],
					step = 0.05,
					min = 0.05,
					max = 2.0,
					isPercent = false,
					get = function()
						return Omen.db.profile.TPSUpdateFrequency
					end,
					set = function(v)
						Omen.db.profile.TPSUpdateFrequency = v
						Omen:ResetTPS()
						Omen:DisableTPSUpdate()
						Omen:EnableTPSUpdate()
					end
				},
				showTPS = {
					type = "toggle",
					desc = L["Show TPS"],
					name = L["Show TPS"],
					get = function()
						return Omen.db.profile.ShowTPS
					end,
					set = function(t)
						Omen:ToggleColumn("tps", t)
						Omen.db.profile.ShowTPS = t
					end
				},
			}
		},
		backdropColor = {
			type = "color",
			hasAlpha = true,
			name = L["Background Color"],
			desc = L["Background Color"],
			get = function()
				local f = Omen.db.profile.BackdropColor
				return f.r, f.g, f.b, f.a
			end,
			set = function(r,g,b,a)
				local f = Omen.db.profile.BackdropColor
				f.r, f.g, f.b, f.a = r, g, b, a
				Omen.Anchor:SetBackdropColor(r, g, b, a)
			end
		},
		backdropBorderColor = {
			type = "color",
			hasAlpha = true,
			name = L["Border Color"],
			desc = L["Border Color"],
			get = function()
				local f = Omen.db.profile.BorderColor
				return f.r, f.g, f.b, f.a
			end,
			set = function(r,g,b,a)
				local f = Omen.db.profile.BorderColor
				f.r, f.g, f.b, f.a = r, g, b, a
				Omen.Anchor:SetBackdropBorderColor(r, g, b, a)
			end
		}		
	}
}

local default_color = {r = 1, g = 0, b = 0}
local barOptions = {
	type='group',
	args = {
		showGraph = {
			name = L["Compare Threat Velocity"],
			desc = L["Compare Threat Velocity"],
			type = "execute",
			func = function()
				Omen:CreateTPSGraph(UnitName("player"), Omen.lastBarClicked.unitname)
			end
		},
		customColor = {
			type = "color",
			hasAlpha = true,
			name = L["Bar Color"],
			desc = L["Bar Color"],
			get = function()
				return Omen.lastBarClicked.tex:GetVertexColor()
			end,
			set = function(r,g,b,a)
				local f = Omen.db.profile.CustomColors[Omen.lastBarClicked.unitname]
				if not f then
					f = {}
					Omen.db.profile.CustomColors[Omen.lastBarClicked.unitname] = f
				end
				f.r, f.g, f.b = r, g, b
				Omen:UpdateBarDisplays()
			end
		},
		defaultColor = {
			type = "execute",
			name = L["Use Default Color"],
			desc = L["Use Default Color"],
			func = function()
				Omen.db.profile.CustomColors[Omen.lastBarClicked.unitname] = nil
				Omen:UpdateBarDisplays()
			end,
			disabled = function()
				return Omen.db.profile.CustomColors[Omen.lastBarClicked.unitname] == nil
			end
		},
		barTexture = {
			type = "text",
			name = L["Bar Texture"],
			desc = L["Bar Texture"],
			get = function() return Omen.db.profile.CustomBarTextures[Omen.lastBarClicked.unitname] or Omen.db.profile.BarTexture end,
			set = function(tex)
				Omen.db.profile.CustomBarTextures[Omen.lastBarClicked.unitname] = tex
				Omen:UpdateBarDisplays()
			end,
			validate = SML_statusbar,
			usage = "<texture>",
		},
		defaultTexture = {
			type = "execute",
			name = L["Use Default Texture"],
			desc = L["Use Default Texture"],
			func = function()
				Omen.db.profile.CustomBarTextures[Omen.lastBarClicked.unitname] = nil
				Omen:UpdateBarDisplays()
			end,
			disabled = function()
				return Omen.db.profile.CustomBarTextures[Omen.lastBarClicked.unitname] == nil
			end	
		},
		barHeight = {
			type = "range",
			name = L["Bar Height"],
			desc = L["Bar Height"],
			step = 1,
			min = 10,
			max = 100,
			isPercent = false,
			get = function()
				return Omen.db.profile.CustomBarHeights[Omen.lastBarClicked.unitname] or Omen.db.profile.BarHeight
			end,
			set = function(h)	
				if h == Omen.db.profile.BarHeight then
					h = nil
				end
				Omen.db.profile.CustomBarHeights[Omen.lastBarClicked.unitname] = h
				Omen:UpdateBarDisplays()
			end
		},
		defaultBarHeight = {
			type = "execute",
			name = L["Use Default Bar Height"],
			desc = L["Use Default Bar Height"],
			func = function()
				Omen.db.profile.CustomBarHeights[Omen.lastBarClicked.unitname] = nil
				Omen:UpdateBarDisplays()
			end,
			disabled = function()
				return Omen.db.profile.CustomBarHeights[Omen.lastBarClicked.unitname] == nil
			end	
		}
	}
}

Omen.OnMenuRequest = options
Omen.barMenu = barOptions
Omen.mainMenu = options
Omen:RegisterChatCommand({"/omen"}, options, "OMEN")